@@ -2,28 +2,26 @@
  * \file AlbersEqualArea.hpp
  * \brief Header for GeographicLib::AlbersEqualArea class
  *
- * Copyright (c) Charles Karney (2010, 2011) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2010-2012) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_ALBERSEQUALAREA_HPP)
-#define GEOGRAPHICLIB_ALBERSEQUALAREA_HPP \
-  "$Id: d17f37d1bec84543dc3753e882d8e95f1c1d5a1b $"
+#define GEOGRAPHICLIB_ALBERSEQUALAREA_HPP 1
 
-#include <algorithm>
 #include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
   /**
-   * \brief Albers Equal Area Conic Projection
+   * \brief Albers equal area conic projection
    *
    * Implementation taken from the report,
    * - J. P. Snyder,
    *   <a href="http://pubs.er.usgs.gov/usgspubs/pp/pp1395"> Map Projections: A
    *   Working Manual</a>, USGS Professional Paper 1395 (1987),
-   *   pp. 101&ndash;102.
+   *   pp. 101--102.
    *
    * This is a implementation of the equations in Snyder except that divided
    * differences will be [have been] used to transform the expressions into
@@ -44,10 +42,10 @@ namespace GeographicLib {
    * longitude) is specified as the \e lon0 argument of the
    * AlbersEqualArea::Forward and AlbersEqualArea::Reverse functions.
    * AlbersEqualArea::Forward and AlbersEqualArea::Reverse also return the
-   * meridian convergence, \e gamma, and azimuthal scale, \e k.  A small square
+   * meridian convergence, &gamma;, and azimuthal scale, \e k.  A small square
    * aligned with the cardinal directions is projected to a rectangle with
    * dimensions \e k (in the E-W direction) and 1/\e k (in the N-S direction).
-   * The E-W sides of the rectangle are oriented \e gamma degrees
+   * The E-W sides of the rectangle are oriented &gamma; degrees
    * counter-clockwise from the \e x axis.  There is no provision in this class
    * for specifying a false easting or false northing or a different latitude
    * of origin.
@@ -59,7 +57,7 @@ namespace GeographicLib {
    * providing access to the functionality of LambertConformalConic and
    * AlbersEqualArea.
    **********************************************************************/
-  class GEOGRAPHIC_EXPORT AlbersEqualArea {
+  class GEOGRAPHICLIB_EXPORT AlbersEqualArea {
   private:
     typedef Math::real real;
     real _a, _f, _fm, _e2, _e, _e2m, _qZ, _qx;
@@ -79,23 +77,30 @@ namespace GeographicLib {
     // x                              if f = 0
     inline real atanhee(real x) const throw() {
       return _f > 0 ? Math::atanh(_e * x)/_e :
-        (_f < 0 ? std::atan(_e * x)/_e : x);
+        // We only invoke atanhee in txif for positive latitude.  Then x is
+        // only negative for very prolate ellipsoids (_b/_a >= sqrt(2)) and we
+        // still need to return a positive result in this case; hence the need
+        // for the call to atan2.
+        (_f < 0 ? (std::atan2(_e * std::abs(x), x < 0 ? -1 : 1)/_e) : x);
     }
     // return atanh(sqrt(x))/sqrt(x) - 1, accurate for small x
     static real atanhxm1(real x) throw();
 
     // Divided differences
     // Definition: Df(x,y) = (f(x)-f(y))/(x-y)
-    // See: W. M. Kahan and R. J. Fateman,
-    // Symbolic computation of divided differences,
-    // SIGSAM Bull. 33(3), 7-28 (1999)
-    // http://doi.acm.org/10.1145/334714.334716
-    // http://www.cs.berkeley.edu/~fateman/papers/divdiff.pdf
+    // See:
+    //   W. M. Kahan and R. J. Fateman,
+    //   Symbolic computation of divided differences,
+    //   SIGSAM Bull. 33(3), 7-28 (1999)
+    //   http://dx.doi.org/10.1145/334714.334716
+    //   http://www.cs.berkeley.edu/~fateman/papers/divdiff.pdf
     //
     // General rules
     // h(x) = f(g(x)): Dh(x,y) = Df(g(x),g(y))*Dg(x,y)
     // h(x) = f(x)*g(x):
-    //        Dh(x,y) = Df(x,y)*(g(x)+g(y))/2 + Dg(x,y)*(f(x)+f(y))/2
+    //        Dh(x,y) = Df(x,y)*g(x) + Dg(x,y)*f(y)
+    //                = Df(x,y)*g(y) + Dg(x,y)*f(x)
+    //                = Df(x,y)*(g(x)+g(y))/2 + Dg(x,y)*(f(x)+f(y))/2
     //
     // sn(x) = x/sqrt(1+x^2): Dsn(x,y) = (x+y)/((sn(x)+sn(y))*(1+x^2)*(1+y^2))
     static inline real Dsn(real x, real y, real sx, real sy) throw() {
@@ -114,6 +119,8 @@ namespace GeographicLib {
     void Init(real sphi1, real cphi1, real sphi2, real cphi2, real k1) throw();
     real txif(real tphi) const throw();
     real tphif(real txi) const throw();
+
+    friend class Ellipsoid;           // For access to txif, tphif, etc.
   public:
 
     /**
@@ -125,9 +132,10 @@ namespace GeographicLib {
      *   to 1/\e f.
      * @param[in] stdlat standard parallel (degrees), the circle of tangency.
      * @param[in] k0 azimuthal scale on the standard parallel.
-     *
-     * An exception is thrown if \e a or \e k0 is not positive or if \e stdlat
-     * is not in the range [-90, 90].
+     * @exception GeographicErr if \e a, (1 &minus; \e f ) \e a, or \e k0 is
+     *   not positive.
+     * @exception GeographicErr if \e stdlat is not in [&minus;90&deg;,
+     *   90&deg;].
      **********************************************************************/
     AlbersEqualArea(real a, real f, real stdlat, real k0);
 
@@ -141,10 +149,11 @@ namespace GeographicLib {
      * @param[in] stdlat1 first standard parallel (degrees).
      * @param[in] stdlat2 second standard parallel (degrees).
      * @param[in] k1 azimuthal scale on the standard parallels.
-     *
-     * An exception is thrown if \e a or \e k0 is not positive or if \e stdlat1
-     * or \e stdlat2 is not in the range [-90, 90].  In addition, an exception
-     * is thrown if \e stdlat1 and \e stdlat2 are opposite poles.
+     * @exception GeographicErr if \e a, (1 &minus; \e f ) \e a, or \e k1 is
+     *   not positive.
+     * @exception GeographicErr if \e stdlat1 or \e stdlat2 is not in
+     *   [&minus;90&deg;, 90&deg;], or if \e stdlat1 and \e stdlat2 are
+     *   opposite poles.
      **********************************************************************/
     AlbersEqualArea(real a, real f, real stdlat1, real stdlat2, real k1);
 
@@ -160,12 +169,17 @@ namespace GeographicLib {
      * @param[in] sinlat2 sine of second standard parallel.
      * @param[in] coslat2 cosine of second standard parallel.
      * @param[in] k1 azimuthal scale on the standard parallels.
+     * @exception GeographicErr if \e a, (1 &minus; \e f ) \e a, or \e k1 is
+     *   not positive.
+     * @exception GeographicErr if \e stdlat1 or \e stdlat2 is not in
+     *   [&minus;90&deg;, 90&deg;], or if \e stdlat1 and \e stdlat2 are
+     *   opposite poles.
      *
      * This allows parallels close to the poles to be specified accurately.
      * This routine computes the latitude of origin and the azimuthal scale at
-     * this latitude.  If \e dlat = abs(\e lat2 - \e lat1) <= 160<sup>o</sup>,
-     * then the error in the latitude of origin is less than
-     * 4.5e-14<sup>o</sup>.
+     * this latitude.  If \e dlat = abs(\e lat2 &minus; \e lat1) &le; 160&deg;,
+     * then the error in the latitude of origin is less than 4.5 &times;
+     * 10<sup>&minus;14</sup>d;.
      **********************************************************************/
     AlbersEqualArea(real a, real f,
                     real sinlat1, real coslat1,
@@ -177,10 +191,11 @@ namespace GeographicLib {
      *
      * @param[in] lat (degrees).
      * @param[in] k azimuthal scale at latitude \e lat (default 1).
+     * @exception GeographicErr \e k is not positive.
+     * @exception GeographicErr if \e lat is not in (&minus;90&deg;,
+     *   90&deg;).
      *
-     * This allows a "latitude of conformality" to be specified.  An exception
-     * is thrown if \e k is not positive or if \e lat is not in the range (-90,
-     * 90).
+     * This allows a "latitude of conformality" to be specified.
      **********************************************************************/
     void SetScale(real lat, real k = real(1));
 
@@ -198,9 +213,10 @@ namespace GeographicLib {
      *
      * The latitude origin is given by AlbersEqualArea::LatitudeOrigin().  No
      * false easting or northing is added and \e lat should be in the range
-     * [-90, 90]; \e lon and \e lon0 should be in the range [-180, 360].  The
-     * values of \e x and \e y returned for points which project to infinity
-     * (i.e., one or both of the poles) will be large but finite.
+     * [&minus;90&deg;, 90&deg;]; \e lon and \e lon0 should be in the
+     * range [&minus;540&deg;, 540&deg;).  The values of \e x and \e y
+     * returned for points which project to infinity (i.e., one or both of the
+     * poles) will be large but finite.
      **********************************************************************/
     void Forward(real lon0, real lat, real lon,
                  real& x, real& y, real& gamma, real& k) const throw();
@@ -219,9 +235,11 @@ namespace GeographicLib {
      *
      * The latitude origin is given by AlbersEqualArea::LatitudeOrigin().  No
      * false easting or northing is added.  \e lon0 should be in the range
-     * [-180, 360].  The value of \e lon returned is in the range [-180, 180).
-     * The value of \e lat returned is in the range [-90,90].  If the input
-     * point is outside the legal projected space the nearest pole is returned.
+     * [&minus;540&deg;, 540&deg;).  The value of \e lon returned is in
+     * the range [&minus;180&deg;, 180&deg;).  The value of \e lat
+     * returned is in the range [&minus;90&deg;, 90&deg;].  If the
+     * input point is outside the legal projected space the nearest pole is
+     * returned.
      **********************************************************************/
     void Reverse(real lon0, real x, real y,
                  real& lat, real& lon, real& gamma, real& k) const throw();
@@ -294,14 +312,14 @@ namespace GeographicLib {
 
     /**
      * A global instantiation of AlbersEqualArea with the WGS84 ellipsoid, \e
-     * stdlat = 90<sup>o</sup>, and \e k0 = 1.  This degenerates to the
+     * stdlat = 90&deg;, and \e k0 = 1.  This degenerates to the
      * Lambert azimuthal equal area projection.
      **********************************************************************/
     static const AlbersEqualArea AzimuthalEqualAreaNorth;
 
     /**
      * A global instantiation of AlbersEqualArea with the WGS84 ellipsoid, \e
-     * stdlat = -90<sup>o</sup>, and \e k0 = 1.  This degenerates to the
+     * stdlat = &minus;90&deg;, and \e k0 = 1.  This degenerates to the
      * Lambert azimuthal equal area projection.
      **********************************************************************/
     static const AlbersEqualArea AzimuthalEqualAreaSouth;
