@@ -7,9 +7,8 @@
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#ifndef GEOGRAPHICLIB_GEOCOORDS_HPP
-#define GEOGRAPHICLIB_GEOCOORDS_HPP \
-  "$Id: e706d3a35c3be0e2beaf39041cac29beb468a5aa $"
+#if !defined(GEOGRAPHICLIB_GEOCOORDS_HPP)
+#define GEOGRAPHICLIB_GEOCOORDS_HPP 1
 
 #include <GeographicLib/UTMUPS.hpp>
 #include <GeographicLib/Constants.hpp>
@@ -47,7 +46,7 @@ namespace GeographicLib {
    * <a href="GeoConvert.1.html">GeoConvert</a> is a command-line utility
    * providing access to the functionality of GeoCoords.
    **********************************************************************/
-  class GEOGRAPHIC_EXPORT GeoCoords {
+  class GEOGRAPHICLIB_EXPORT GeoCoords {
   private:
     typedef Math::real real;
     real _lat, _long, _easting, _northing, _gamma, _k;
@@ -63,8 +62,8 @@ namespace GeographicLib {
       _alt_k = _k;
       _alt_zone = _zone;
     }
-    void UTMUPSString(int zone, real easting, real northing,
-                      int prec, std::string& utm) const;
+    static void UTMUPSString(int zone, bool northp, real easting, real northing,
+                             int prec, std::string& utm);
     void FixHemisphere();
   public:
 
@@ -72,8 +71,8 @@ namespace GeographicLib {
      **********************************************************************/
     ///@{
     /**
-     * The default constructor is equivalent to \e latitude = 90<sup>o</sup>,
-     * \e longitude = 0<sup>o</sup>.
+     * The default constructor is equivalent to \e latitude = 90&deg;,
+     * \e longitude = 0&deg;.
      **********************************************************************/
     GeoCoords() throw()
       // This is the N pole
@@ -94,6 +93,7 @@ namespace GeographicLib {
      *   below).
      * @param[in] swaplatlong governs the interpretation of geographic
      *   coordinates (see below).
+     * @exception GeographicErr if the \e s is malformed (see below).
      *
      * Parse as a string and interpret it as a geographic position.  The input
      * string is broken into space (or comma) separated pieces and Basic
@@ -117,11 +117,11 @@ namespace GeographicLib {
      *   -  38N 339188 3701405
      *   -  897039 3708229 37N
      *
-     * Latitude and Longitude parsing.  Latitude precedes longitude, unless a
-     * N, S, E, W hemisphere designator is used on one or both coordinates.  If
-     * \e swaplatlong = true (default is false), then longitude precedes
-     * latitude in the absence of a hemisphere designator.  Thus (with \e
-     * swaplatlong = false)
+     * <b>Latitude and Longitude parsing</b>: Latitude precedes longitude,
+     * unless a N, S, E, W hemisphere designator is used on one or both
+     * coordinates.  If \e swaplatlong = true (default is false), then
+     * longitude precedes latitude in the absence of a hemisphere designator.
+     * Thus (with \e swaplatlong = false)
      * - 40 -75
      * - N40 W75
      * - -75 N40
@@ -131,10 +131,12 @@ namespace GeographicLib {
      * are all the same position.  The coordinates may be given in
      * decimal degrees, degrees and decimal minutes, degrees, minutes,
      * seconds, etc.  Use d, ', and &quot; to mark off the degrees,
-     * minutes and seconds.  Alternatively, use : to separate these
-     * components.  Thus
+     * minutes and seconds.  Various alternative symbols for degrees, minutes,
+     * and seconds are allowed.  Alternatively, use : to separate these
+     * components.  (See DMS::Decode for details.)  Thus
      * - 40d30'30&quot;
      * - 40d30'30
+     * - 40&deg;30'30
      * - 40d30.5'
      * - 40d30.5
      * - 40:30:30
@@ -142,18 +144,20 @@ namespace GeographicLib {
      * - 40.508333333
      * .
      * all specify the same angle.  The leading sign applies to all components
-     * so -1d30 is -(1+30/60) = -1.5.  Latitudes must be in the range [-90, 90]
-     * and longitudes in the range [-180, 360].  Internally longitudes are
-     * reduced to the range [-180, 180).
+     * so -1d30 is -(1+30/60) = -1.5.  Latitudes must be in the range
+     * [&minus;90&deg;, 90&deg;] and longitudes in the range
+     * [&minus;540&deg;, 540&deg;).  Internally longitudes are reduced
+     * to the range [&minus;180&deg;, 180&deg;).
      *
-     * UTM/UPS parsing.  For UTM zones (-80 <= Lat <= 84), the zone designator
-     * is made up of a zone number (for 1 to 60) and a hemisphere letter (N or
-     * S), e.g., 38N.  The latitude zone designer ([C&ndash;M] in the southern
-     * hemisphere and [N&ndash;X] in the northern) should NOT be used.  (This
-     * is part of the MGRS coordinate.)  The zone designator for the poles
-     * (where UPS is employed) is a hemisphere letter by itself, i.e., N or S.
+     * <b>UTM/UPS parsing</b>: For UTM zones (&minus;80&deg; &le; Lat <
+     * 84&deg;), the zone designator is made up of a zone number (for 1 to 60)
+     * and a hemisphere letter (N or S), e.g., 38N.  The latitude zone designer
+     * ([C--M] in the southern hemisphere and [N--X] in the northern) should
+     * NOT be used.  (This is part of the MGRS coordinate.)  The zone
+     * designator for the poles (where UPS is employed) is a hemisphere letter
+     * by itself, i.e., N or S.
      *
-     * MGRS parsing interprets the grid references as square area at the
+     * <b>MGRS parsing</b> interprets the grid references as square area at the
      * specified precision (1m, 10m, 100m, etc.).  If \e centerp = true (the
      * default), the center of this square is then taken to be the precise
      * position; thus:
@@ -177,6 +181,11 @@ namespace GeographicLib {
      * @param[in] longitude (degrees).
      * @param[in] zone if specified, force the UTM/UPS representation to use a
      *   specified zone using the rules given in UTMUPS::zonespec.
+     * @exception GeographicErr if \e latitude is not in [&minus;90&deg;,
+     *   90&deg;].
+     * @exception GeographicErr if \e longitude is not in [&minus;540&deg;,
+     *   540&deg;).
+     * @exception GeographicErr if \e zone cannot be used for this location.
      **********************************************************************/
     GeoCoords(real latitude, real longitude, int zone = UTMUPS::STANDARD) {
       Reset(latitude, longitude, zone);
@@ -189,6 +198,8 @@ namespace GeographicLib {
      * @param[in] northp hemisphere (true means north, false means south).
      * @param[in] easting (meters).
      * @param[in] northing (meters).
+     * @exception GeographicErr if \e zone, \e easting, or \e northing is
+     *   outside its allowed range.
      **********************************************************************/
     GeoCoords(int zone, bool northp, real easting, real northing) {
       Reset(zone, northp, easting, northing);
@@ -197,6 +208,13 @@ namespace GeographicLib {
     /**
      * Reset the location from a string.  See
      * GeoCoords(const std::string& s, bool centerp, bool swaplatlong).
+     *
+     * @param[in] s 1-element, 2-element, or 3-element string representation of
+     *   the position.
+     * @param[in] centerp governs the interpretation of MGRS coordinates.
+     * @param[in] swaplatlong governs the interpretation of geographic
+     *   coordinates.
+     * @exception GeographicErr if the \e s is malformed.
      **********************************************************************/
     void Reset(const std::string& s,
                bool centerp = true, bool swaplatlong = false);
@@ -204,6 +222,16 @@ namespace GeographicLib {
     /**
      * Reset the location in terms of geographic coordinates.  See
      * GeoCoords(real latitude, real longitude, int zone).
+     *
+     * @param[in] latitude (degrees).
+     * @param[in] longitude (degrees).
+     * @param[in] zone if specified, force the UTM/UPS representation to use a
+     *   specified zone using the rules given in UTMUPS::zonespec.
+     * @exception GeographicErr if \e latitude is not in [&minus;90&deg;,
+     *   90&deg;].
+     * @exception GeographicErr if \e longitude is not in [&minus;540&deg;,
+     *   540&deg;).
+     * @exception GeographicErr if \e zone cannot be used for this location.
      **********************************************************************/
     void Reset(real latitude, real longitude, int zone = UTMUPS::STANDARD) {
       UTMUPS::Forward(latitude, longitude,
@@ -211,14 +239,21 @@ namespace GeographicLib {
                       zone);
       _lat = latitude;
       _long = longitude;
-      if (_long >= 180)
-        _long -= 360;
+      if (_long >= 180) _long -= 360;
+      else if (_long < -180) _long += 360;
       CopyToAlt();
     }
 
     /**
      * Reset the location in terms of UPS/UPS coordinates.  See
      * GeoCoords(int zone, bool northp, real easting, real northing).
+     *
+     * @param[in] zone UTM zone (zero means UPS).
+     * @param[in] northp hemisphere (true means north, false means south).
+     * @param[in] easting (meters).
+     * @param[in] northing (meters).
+     * @exception GeographicErr if \e zone, \e easting, or \e northing is
+     *   outside its allowed range.
      **********************************************************************/
     void Reset(int zone, bool northp, real easting, real northing) {
       UTMUPS::Reverse(zone, northp, easting, northing,
@@ -289,6 +324,7 @@ namespace GeographicLib {
      * Specify alternate zone number.
      *
      * @param[in] zone zone number for the alternate representation.
+     * @exception GeographicErr if \e zone cannot be used for this location.
      *
      * See UTMUPS::zonespec for more information on the interpretation of \e
      * zone.  Note that \e zone == UTMUPS::STANDARD (the default) use the
@@ -346,13 +382,14 @@ namespace GeographicLib {
      *
      * @param[in] prec precision (relative to about 1m).
      * @param[in] swaplatlong if true give longitude first (default = false)
+     * @exception std::bad_alloc if memory for the string can't be allocated.
      * @return decimal latitude/longitude string representation.
      *
      * Precision specifies accuracy of representation as follows:
-     * - prec = -5 (min), 1d
-     * - prec = 0, 10<sup>-5</sup>d (about 1m)
-     * - prec = 3, 10<sup>-8</sup>d
-     * - prec = 9 (max), 10<sup>-14</sup>d
+     * - prec = &minus;5 (min), 1&deg;
+     * - prec = 0, 10<sup>&minus;5</sup>&deg; (about 1m)
+     * - prec = 3, 10<sup>&minus;8</sup>&deg;
+     * - prec = 9 (max), 10<sup>&minus;14</sup>&deg;
      **********************************************************************/
     std::string GeoRepresentation(int prec = 0, bool swaplatlong = false) const;
 
@@ -364,57 +401,43 @@ namespace GeographicLib {
      * @param[in] swaplatlong if true give longitude first (default = false)
      * @param[in] dmssep if non-null, use as the DMS separator character
      *   (instead of d, ', &quot; delimiters).
+     * @exception std::bad_alloc if memory for the string can't be allocated.
      * @return DMS latitude/longitude string representation.
      *
      * Precision specifies accuracy of representation as follows:
-     * - prec = -5 (min), 1d
-     * - prec = -4, 0.1d
-     * - prec = -3, 1'
-     * - prec = -2, 0.1'
-     * - prec = -1, 1&quot;
+     * - prec = &minus;5 (min), 1&deg;
+     * - prec = &minus;4, 0.1&deg;
+     * - prec = &minus;3, 1'
+     * - prec = &minus;2, 0.1'
+     * - prec = &minus;1, 1&quot;
      * - prec = 0, 0.1&quot; (about 3m)
      * - prec = 1, 0.01&quot;
-     * - prec = 10 (max), 10<sup>-11</sup>&quot;
+     * - prec = 10 (max), 10<sup>&minus;11</sup>&quot;
      **********************************************************************/
-    std::string DMSRepresentation(int prec, bool swaplatlong, char dmssep)
+    std::string DMSRepresentation(int prec = 0, bool swaplatlong = false,
+                                  char dmssep = char(0))
       const;
 
-    /**
-     * String representation with latitude and longitude as degrees, minutes,
-     * seconds, and hemisphere.
-     *
-     * @param[in] prec precision (relative to about 1m)
-     * @param[in] swaplatlong if true give longitude first (default = false)
-     * @return DMS latitude/longitude string representation.
-     *
-     * <b>COMPATIBILITY NOTE:</b> This function calls
-     * DMSRepresentation(int, bool, char) const with a 3rd argument of
-     * char(0).  At some point, DMSRepresentation(int, bool) const and
-     * will be withdrawn and the interface to
-     * DMSRepresentation(int, bool, char) const changed so that its
-     * arguments have default values.  This will preserve source-level
-     * compatibility.
-     **********************************************************************/
-    std::string DMSRepresentation(int prec = 0, bool swaplatlong = false) const;
-
     /**
      * MGRS string.
      *
      * @param[in] prec precision (relative to about 1m).
+     * @exception std::bad_alloc if memory for the string can't be allocated.
      * @return MGRS string.
      *
      * This gives the coordinates of the enclosing grid square with size given
      * by the precision.  Thus 38N 444180 3684790 converted to a MGRS
-     * coordinate at precision -2 (100m) is 38SMB441847 and not 38SMB442848.
-     * \e prec specifies the precision of the MGRS string as follows:
-     * - prec = -5 (min), 100km
-     * - prec = -4, 10km
-     * - prec = -3, 1km
-     * - prec = -2, 100m
-     * - prec = -1, 10m
+     * coordinate at precision &minus;2 (100m) is 38SMB441847 and not
+     * 38SMB442848.  \e prec specifies the precision of the MGRS string as
+     * follows:
+     * - prec = &minus;5 (min), 100km
+     * - prec = &minus;4, 10km
+     * - prec = &minus;3, 1km
+     * - prec = &minus;2, 100m
+     * - prec = &minus;1, 10m
      * - prec = 0, 1m
      * - prec = 1, 0.1m
-     * - prec = 6 (max), 1um
+     * - prec = 6 (max), 1&mu;m
      **********************************************************************/
     std::string MGRSRepresentation(int prec = 0) const;
 
@@ -422,29 +445,65 @@ namespace GeographicLib {
      * UTM/UPS string.
      *
      * @param[in] prec precision (relative to about 1m)
+     * @exception std::bad_alloc if memory for the string can't be allocated.
      * @return UTM/UPS string representation: zone designator, easting, and
      *   northing.
      *
      * Precision specifies accuracy of representation as follows:
-     * - prec = -5 (min), 100km
-     * - prec = -3, 1km
+     * - prec = &minus;5 (min), 100km
+     * - prec = &minus;3, 1km
      * - prec = 0, 1m
      * - prec = 3, 1mm
-     * - prec = 6, 1um
+     * - prec = 6, 1&mu;m
      * - prec = 9 (max), 1nm
      **********************************************************************/
     std::string UTMUPSRepresentation(int prec = 0) const;
 
+    /**
+     * UTM/UPS string with hemisphere override.
+     *
+     * @param[in] prec precision (relative to about 1m)
+     * @param[in] northp hemisphere override
+     * @exception GeographicErr if the hemisphere override attempts to change
+     *   UPS N to UPS S or vice verse.
+     * @exception std::bad_alloc if memory for the string can't be allocated.
+     * @return UTM/UPS string representation: zone designator, easting, and
+     *   northing.
+     **********************************************************************/
+    std::string UTMUPSRepresentation(bool northp, int prec = 0) const;
+
     /**
      * MGRS string for the alternate zone.  See GeoCoords::MGRSRepresentation.
+     *
+     * @param[in] prec precision (relative to about 1m).
+     * @exception std::bad_alloc if memory for the string can't be allocated.
+     * @return MGRS string.
      **********************************************************************/
     std::string AltMGRSRepresentation(int prec = 0) const;
 
     /**
      * UTM/UPS string for the alternate zone.  See
      * GeoCoords::UTMUPSRepresentation.
+     *
+     * @param[in] prec precision (relative to about 1m)
+     * @exception std::bad_alloc if memory for the string can't be allocated.
+     * @return UTM/UPS string representation: zone designator, easting, and
+     *   northing.
      **********************************************************************/
     std::string AltUTMUPSRepresentation(int prec = 0) const;
+
+    /**
+     * UTM/UPS string for the alternate zone, with hemisphere override.
+     *
+     * @param[in] prec precision (relative to about 1m)
+     * @param[in] northp hemisphere override
+     * @exception GeographicErr if the hemisphere override attempts to change
+     *   UPS N to UPS S or vice verse.
+     * @exception std::bad_alloc if memory for the string can't be allocated.
+     * @return UTM/UPS string representation: zone designator, easting, and
+     *   northing.
+     **********************************************************************/
+    std::string AltUTMUPSRepresentation(bool northp, int prec = 0) const;
     ///@}
 
     /** \name Inspector functions
