@@ -8,29 +8,27 @@
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_GEOID_HPP)
-#define GEOGRAPHICLIB_GEOID_HPP \
-  "$Id: 4e4eb5941d16ad00416798703d246a6f7ef5fe46 $"
+#define GEOGRAPHICLIB_GEOID_HPP 1
 
-#include <string>
 #include <vector>
 #include <fstream>
 #include <GeographicLib/Constants.hpp>
 
 #if defined(_MSC_VER)
-// Squelch warnings about dll vs vector
-#pragma warning (push)
-#pragma warning (disable: 4251)
+// Squelch warnings about dll vs vector and constant conditional expressions
+#  pragma warning (push)
+#  pragma warning (disable: 4251 4127)
 #endif
 
 #if !defined(PGM_PIXEL_WIDTH)
 /**
- * The size of the pixel data in the pgm data files for the geoids.  2
- * is the standard size corresponding to a maxval 2^16-1.  Setting it
- * to 4 uses a maxval of 2^32-1 and changes the extension for the data
- * files from .pgm to .pgm4.  Note that the format of these pgm4 files
+ * The size of the pixel data in the pgm data files for the geoids.  2 is the
+ * standard size corresponding to a maxval 2<sup>16</sup>&minus;1.  Setting it
+ * to 4 uses a maxval of 2<sup>32</sup>&minus;1 and changes the extension for
+ * the data files from .pgm to .pgm4.  Note that the format of these pgm4 files
  * is a non-standard extension of the pgm format.
  **********************************************************************/
-#define PGM_PIXEL_WIDTH 2
+#  define PGM_PIXEL_WIDTH 2
 #endif
 
 namespace GeographicLib {
@@ -82,7 +80,7 @@ namespace GeographicLib {
    * providing access to the functionality of Geoid.
    **********************************************************************/
 
-  class GEOGRAPHIC_EXPORT Geoid {
+  class GEOGRAPHICLIB_EXPORT Geoid {
   private:
     typedef Math::real real;
 #if PGM_PIXEL_WIDTH != 4
@@ -211,15 +209,17 @@ namespace GeographicLib {
      *   true (the default) means cubic.
      * @param[in] threadsafe (optional), if true, construct a thread safe
      *   object.  The default is false
+     * @exception GeographicErr if the data file cannot be found, is
+     *   unreadable, or is corrupt.
+     * @exception GeographicErr if \e threadsafe is true but the memory
+     *   necessary for caching the data can't be allocated.
      *
      * The data file is formed by appending ".pgm" to the name.  If \e path is
      * specified (and is non-empty), then the file is loaded from directory, \e
-     * path.  Otherwise the path is given by DefaultGeoidPath().  This may
-     * throw an exception because the file does not exist, is unreadable, or is
-     * corrupt.  If the \e threadsafe parameter is true, the data set is read
-     * into memory (which this may also cause an exception to be thrown), the
-     * data file is closed, and single-cell caching is turned off; this results
-     * in a Geoid object which \e is thread safe.
+     * path.  Otherwise the path is given by DefaultGeoidPath().  If the \e
+     * threadsafe parameter is true, the data set is read into memory, the data
+     * file is closed, and single-cell caching is turned off; this results in a
+     * Geoid object which \e is thread safe.
      **********************************************************************/
     explicit Geoid(const std::string& name, const std::string& path = "",
                    bool cubic = true, bool threadsafe = false);
@@ -231,29 +231,33 @@ namespace GeographicLib {
      * @param[in] west longitude (degrees) of the west edge of the cached area.
      * @param[in] north latitude (degrees) of the north edge of the cached area.
      * @param[in] east longitude (degrees) of the east edge of the cached area.
+     * @exception GeographicErr if the memory necessary for caching the data
+     *   can't be allocated (in this case, you will have no cache and can try
+     *   again with a smaller area).
+     * @exception GeographicErr if there's a problem reading the data.
+     * @exception GeographicErr if this is called on a threadsafe Geoid.
      *
      * Cache the data for the specified "rectangular" area bounded by the
      * parallels \e south and \e north and the meridians \e west and \e east.
      * \e east is always interpreted as being east of \e west, if necessary by
-     * adding 360<sup>o</sup> to its value.  This may throw an error because of
-     * insufficient memory or because of an error reading the data from the
-     * file.  In this case, you can catch the error and either do nothing (you
-     * will have no cache in this case) or try again with a smaller area.  \e
-     * south and \e north should be in the range [-90, 90]; \e west and \e east
-     * should be in the range [-180, 360].  An exception is thrown if this
-     * routine is called on a thread safe Geoid.
+     * adding 360&deg; to its value.  \e south and \e north should be in
+     * the range [&minus;90&deg;, 90&deg;]; \e west and \e east should
+     * be in the range [&minus;540&deg;, 540&deg;).
      **********************************************************************/
     void CacheArea(real south, real west, real north, real east) const;
 
     /**
-     * Cache all the data.  On most computers, this is fast for data sets with
-     * grid resolution of 5' or coarser.  For a 1' grid, the required RAM is
-     * 450MB; a 2.5' grid needs 72MB; and a 5' grid needs 18MB.  This may throw
-     * an error because of insufficient memory or because of an error reading
-     * the data from the file.  In this case, you can catch the error and
-     * either do nothing (you will have no cache in this case) or try using
-     * Geoid::CacheArea on a specific area.  An exception is thrown if this
-     * routine is called on a thread safe Geoid.
+     * Cache all the data.
+     *
+     * @exception GeographicErr if the memory necessary for caching the data
+     *   can't be allocated (in this case, you will have no cache and can try
+     *   again with a smaller area).
+     * @exception GeographicErr if there's a problem reading the data.
+     * @exception GeographicErr if this is called on a threadsafe Geoid.
+     *
+     * On most computers, this is fast for data sets with grid resolution of 5'
+     * or coarser.  For a 1' grid, the required RAM is 450MB; a 2.5' grid needs
+     * 72MB; and a 5' grid needs 18MB.
      **********************************************************************/
     void CacheAll() const { CacheArea(real(-90), real(0),
                                       real(90), real(360)); }
@@ -274,12 +278,12 @@ namespace GeographicLib {
      *
      * @param[in] lat latitude of the point (degrees).
      * @param[in] lon longitude of the point (degrees).
+     * @exception GeographicErr if there's a problem reading the data; this
+     *   never happens if (\e lat, \e lon) is within a successfully cached area.
      * @return geoid height (meters).
      *
-     * The latitude should be in [-90, 90] and longitude should be in
-     * [-180,360].  This may throw an error because of an error reading data
-     * from disk.  However, it will not throw if (\e lat, \e lon) is within a
-     * successfully cached area.
+     * The latitude should be in [&minus;90&deg;, 90&deg;] and
+     * longitude should be in [&minus;540&deg;, 540&deg;).
      **********************************************************************/
     Math::real operator()(real lat, real lon) const {
       real gradn, grade;
@@ -293,17 +297,17 @@ namespace GeographicLib {
      * @param[in] lon longitude of the point (degrees).
      * @param[out] gradn northerly gradient (dimensionless).
      * @param[out] grade easterly gradient (dimensionless).
+     * @exception GeographicErr if there's a problem reading the data; this
+     *   never happens if (\e lat, \e lon) is within a successfully cached area.
      * @return geoid height (meters).
      *
-     * The latitude should be in [-90, 90] and longitude should be in [-180,
-     * 360].  This may throw an error because of an error reading data from
-     * disk.  However, it will not throw if (\e lat, \e lon) is within a
-     * successfully cached area.  As a result of the way that the geoid data is
-     * stored, the calculation of gradients can result in large quantization
-     * errors.  This is particularly acute for fine grids, at high latitudes,
-     * and for the easterly gradient.  If you need to compute the direction of
-     * the acceleration due to gravity accurately, you should use
-     * GravityModel::Gravity.
+     * The latitude should be in [&minus;90&deg;, 90&deg;] and
+     * longitude should be in [&minus;540&deg;, 540&deg;).  As a result
+     * of the way that the geoid data is stored, the calculation of gradients
+     * can result in large quantization errors.  This is particularly acute for
+     * fine grids, at high latitudes, and for the easterly gradient.  If you
+     * need to compute the direction of the acceleration due to gravity
+     * accurately, you should use GravityModel::Gravity.
      **********************************************************************/
     Math::real operator()(real lat, real lon, real& gradn, real& grade) const {
       return height(lat, lon, true, gradn, grade);
@@ -320,6 +324,8 @@ namespace GeographicLib {
      *   conversion; Geoid::GEOIDTOELLIPSOID means convert a height above the
      *   geoid to a height above the ellipsoid; Geoid::ELLIPSOIDTOGEOID means
      *   convert a height above the ellipsoid to a height above the geoid.
+     * @exception GeographicErr if there's a problem reading the data; this
+     *   never happens if (\e lat, \e lon) is within a successfully cached area.
      * @return converted height (meters).
      **********************************************************************/
     Math::real ConvertHeight(real lat, real lon, real h,
@@ -371,7 +377,7 @@ namespace GeographicLib {
      *   (meters).
      *
      * This relies on the value being stored in the data file.  If the value is
-     * absent, return -1.
+     * absent, return &minus;1.
      **********************************************************************/
     Math::real MaxError() const throw() { return _maxerror; }
 
@@ -380,7 +386,7 @@ namespace GeographicLib {
      *   (meters).
      *
      * This relies on the value being stored in the data file.  If the value is
-     * absent, return -1.
+     * absent, return &minus;1.
      **********************************************************************/
     Math::real RMSError() const throw() { return _rmserror; }
 
@@ -498,7 +504,7 @@ namespace GeographicLib {
 } // namespace GeographicLib
 
 #if defined(_MSC_VER)
-#pragma warning (pop)
+#  pragma warning (pop)
 #endif
 
 #endif  // GEOGRAPHICLIB_GEOID_HPP
