@@ -8,16 +8,14 @@
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_UTMUPS_HPP)
-#define GEOGRAPHICLIB_UTMUPS_HPP \
-  "$Id: a529ed8aeaeffb02994254bbc7eb1209aa41b9ca $"
+#define GEOGRAPHICLIB_UTMUPS_HPP 1
 
-#include <sstream>
 #include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
   /**
-   * \brief Convert between Geographic coordinates and UTM/UPS
+   * \brief Convert between geographic coordinates and UTM/UPS
    *
    * UTM and UPS are defined
    * - J. W. Hager, J. F. Behensky, and B. W. Drew,
@@ -60,7 +58,7 @@ namespace GeographicLib {
    * Example of use:
    * \include example-UTMUPS.cpp
    **********************************************************************/
-  class GEOGRAPHIC_EXPORT UTMUPS {
+  class GEOGRAPHICLIB_EXPORT UTMUPS {
   private:
     typedef Math::real real;
     static const real falseeasting_[4];
@@ -69,6 +67,12 @@ namespace GeographicLib {
     static const real maxeasting_[4];
     static const real minnorthing_[4];
     static const real maxnorthing_[4];
+    static const int epsg01N = 32601; // EPSG code for UTM 01N
+    static const int epsg60N = 32660; // EPSG code for UTM 60N
+    static const int epsgN   = 32661; // EPSG code for UPS   N
+    static const int epsg01S = 32701; // EPSG code for UTM 01S
+    static const int epsg60S = 32760; // EPSG code for UTM 60S
+    static const int epsgS   = 32761; // EPSG code for UPS   S
     static real CentralMeridian(int zone) throw()
     { return real(6 * zone - 183); }
     static void CheckLatLon(real lat, real lon);
@@ -82,11 +86,12 @@ namespace GeographicLib {
 
     /**
      * In this class we bring together the UTM and UPS coordinates systems.
-     * The UTM divides the earth between latitudes -80 and 84 into 60 zones
-     * numbered 1 thru 60.  Zone assign zone number 0 to the UPS regions,
-     * covering the two poles.  Within UTMUPS, non-negative zone numbers refer
-     * to one of the "physical" zones, 0 for UPS and [1, 60] for UTM.  Negative
-     * "pseudo-zone" numbers are used to select one of the physical zones.
+     * The UTM divides the earth between latitudes &minus;80&deg; and 84&deg;
+     * into 60 zones numbered 1 thru 60.  Zone assign zone number 0 to the UPS
+     * regions, covering the two poles.  Within UTMUPS, non-negative zone
+     * numbers refer to one of the "physical" zones, 0 for UPS and [1, 60] for
+     * UTM.  Negative "pseudo-zone" numbers are used to select one of the
+     * physical zones.
      **********************************************************************/
     enum zonespec {
       /**
@@ -105,17 +110,17 @@ namespace GeographicLib {
       /**
        * Apply the standard rules for UTM zone assigment extending the UTM zone
        * to each pole to give a zone number in [1, 60].  For example, use UTM
-       * zone 38 for longitude in [42, 48).  The rules include the Norway and
-       * Svalbard exceptions.
+       * zone 38 for longitude in [42&deg;, 48&deg;).  The rules include the
+       * Norway and Svalbard exceptions.
        **********************************************************************/
       UTM = -2,
       /**
        * Apply the standard rules for zone assignment to give a zone number in
-       * [0, 60].  If the latitude is not in [-80, 84), then use UTMUPS::UPS =
-       * 0, otherwise apply the rules for UTMUPS::UTM.  The tests on latitudes
-       * and longitudes are all closed on the lower end open on the upper.
-       * Thus for UTM zone 38, latitude is in [-80, 84) and longitude is in
-       * [42, 48).
+       * [0, 60].  If the latitude is not in [&minus;80&deg;, 84&deg;), then
+       * use UTMUPS::UPS = 0, otherwise apply the rules for UTMUPS::UTM.  The
+       * tests on latitudes and longitudes are all closed on the lower end open
+       * on the upper.  Thus for UTM zone 38, latitude is in [&minus;80&deg;,
+       * 84&deg;) and longitude is in [42&deg;, 48&deg;).
        **********************************************************************/
       STANDARD = -1,
       /**
@@ -149,12 +154,14 @@ namespace GeographicLib {
      *
      * @param[in] lat latitude (degrees).
      * @param[in] lon longitude (degrees).
-     * @param[in] setzone zone override (optional).
+     * @param[in] setzone zone override (optional).  If omitted, use the
+     *   standard rules for picking the zone.  If \e setzone is given then use
+     *   that zone if it is non-negative, otherwise apply the rules given in
+     *   UTMUPS::zonespec.
+     * @exception GeographicErr if \e setzone is outside the range
+     *   [UTMUPS::MINPSEUDOZONE, UTMUPS::MAXZONE] = [&minus;4, 60].
      *
-     * This is exact.  If the optional argument \e setzone is given then use
-     * that zone if it is non-negative, otherwise apply the rules given in
-     * UTMUPS::zonespec.  Throws an error if \e setzone is outsize the range
-     * [UTMUPS::MINPSEUDOZONE, UTMUPS::MAXZONE] = [-4, 60].
+     * This is exact.
      **********************************************************************/
     static int StandardZone(real lat, real lon, int setzone = STANDARD);
 
@@ -169,15 +176,35 @@ namespace GeographicLib {
      * @param[out] y northing of point (meters).
      * @param[out] gamma meridian convergence at point (degrees).
      * @param[out] k scale of projection at point.
-     * @param[in] setzone zone override.
+     * @param[in] setzone zone override (optional).
      * @param[in] mgrslimits if true enforce the stricter MGRS limits on the
      *   coordinates (default = false).
+     * @exception GeographicErr if \e lat is not in [&minus;90&deg;,
+     *   90&deg;].
+     * @exception GeographicErr if \e lon is not in [&minus;540&deg;,
+     *   540&deg;).
+     * @exception GeographicErr if the resulting \e x or \e y is out of allowed
+     *   range (see Reverse); in this case, these arguments are unchanged.
      *
-     * The preferred zone for the result can be specified with \e setzone, see
-     * UTMUPS::StandardZone.  Throw error if the resulting easting or northing
-     * is outside the allowed range (see Reverse), in which case the arguments
-     * are unchanged.  This also returns meridian convergence \e gamma
-     * (degrees) and scale \e k.  The accuracy of the conversion is about 5nm.
+     * If \e setzone is omitted, use the standard rules for picking the zone.
+     * If \e setzone is given then use that zone if it is non-negative,
+     * otherwise apply the rules given in UTMUPS::zonespec.  The accuracy of
+     * the conversion is about 5nm.
+     *
+     * The northing \e y jumps by UTMUPS::UTMShift() when crossing the equator
+     * in the southerly direction.  Sometimes it is useful to remove this
+     * discontinuity in \e y by extending the "northern" hemisphere using
+     * UTMUPS::Transfer:
+     * \code
+     double lat = -1, lon = 123;
+     int zone;
+     bool northp;
+     double x, y, gamma, k;
+     GeographicLib::UTMUPS::Forward(lat, lon, zone, northp, x, y, gamma, k);
+     GeographicLib::UTMUPS::Transfer(zone, northp, x, y,
+                                     zone, true,   x, y, zone);
+     northp = true;
+     \endcode
      **********************************************************************/
     static void Forward(real lat, real lon,
                         int& zone, bool& northp, real& x, real& y,
@@ -197,17 +224,17 @@ namespace GeographicLib {
      * @param[out] k scale of projection at point.
      * @param[in] mgrslimits if true enforce the stricter MGRS limits on the
      *   coordinates (default = false).
+     * @exception GeographicErr if \e zone, \e x, or \e y is out of allowed
+     *   range; this this case the arguments are unchanged.
      *
-     * Throw error if easting or northing is outside the allowed range (see
-     * below), in which case the arguments are unchanged.  The accuracy of the
-     * conversion is about 5nm.
+     * The accuracy of the conversion is about 5nm.
      *
      * UTM eastings are allowed to be in the range [0km, 1000km], northings are
      * allowed to be in in [0km, 9600km] for the northern hemisphere and in
-     * [900km, 10000km] for the southern hemisphere.  (However UTM northings
+     * [900km, 10000km] for the southern hemisphere.  However UTM northings
      * can be continued across the equator.  So the actual limits on the
      * northings are [-9100km, 9600km] for the "northern" hemisphere and
-     * [900km, 19600km] for the "southern" hemisphere.)
+     * [900km, 19600km] for the "southern" hemisphere.
      *
      * UPS eastings and northings are allowed to be in the range [1200km,
      * 2800km] in the northern hemisphere and in [700km, 3100km] in the
@@ -244,12 +271,50 @@ namespace GeographicLib {
       Reverse(zone, northp, x, y, lat, lon, gamma, k, mgrslimits);
     }
 
+    /**
+     * Transfer UTM/UPS coordinated from one zone to another.
+     *
+     * @param[in] zonein the UTM zone for \e xin and \e yin (or zero for UPS).
+     * @param[in] northpin hemisphere for \e xin and \e yin (true means north,
+     *   false means south).
+     * @param[in] xin easting of point (meters) in \e zonein.
+     * @param[in] yin northing of point (meters) in \e zonein.
+     * @param[in] zoneout the requested UTM zone for \e xout and \e yout (or
+     *   zero for UPS).
+     * @param[in] northpout hemisphere for \e xout output and \e yout.
+     * @param[out] xout easting of point (meters) in \e zoneout.
+     * @param[out] yout northing of point (meters) in \e zoneout.
+     * @param[out] zone the actual UTM zone for \e xout and \e yout (or zero
+     *   for UPS); this equals \e zoneout if \e zoneout &ge; 0.
+     * @exception GeographicErr if \e zonein is out of range (see below).
+     * @exception GeographicErr if \e zoneout is out of range (see below).
+     * @exception GeographicErr if \e xin or \e yin fall outside their allowed
+     *   ranges (see UTMUPS::Reverse).
+     * @exception GeographicErr if \e xout or \e yout fall outside their
+     *   allowed ranges (see UTMUPS::Reverse).
+     *
+     * \e zonein must be in the range [UTMUPS::MINZONE, UTMUPS::MAXZONE] = [0,
+     * 60] with \e zonein = UTMUPS::UPS, 0, indicating UPS.  \e zonein may
+     * also be UTMUPS::INVALID.
+     *
+     * \e zoneout must be in the range [UTMUPS::MINPSEUDOZONE, UTMUPS::MAXZONE]
+     * = [-4, 60].  If \e zoneout &lt; UTMUPS::MINZONE then the rules give in
+     * the documentation of UTMUPS::zonespec are applied, and \e zone is set to
+     * the actual zone used for output.
+     *
+     * (\e xout, \e yout) can overlap with (\e xin, \e yin).
+     **********************************************************************/
+    static void Transfer(int zonein, bool northpin, real xin, real yin,
+                         int zoneout, bool northpout, real& xout, real& yout,
+                         int& zone);
+
     /**
      * Decode a UTM/UPS zone string.
      *
      * @param[in] zonestr string representation of zone and hemisphere.
      * @param[out] zone the UTM zone (zero means UPS).
      * @param[out] northp hemisphere (true means north, false means south).
+     * @exception GeographicErr if \e zonestr is malformed.
      *
      * For UTM, \e zonestr has the form of a zone number in the range
      * [UTMUPS::MINUTMZONE, UTMUPS::MAXUTMZONE] = [1, 60] followed by a
@@ -258,15 +323,17 @@ namespace GeographicLib {
      * well that "38S" indicates the southern hemisphere of zone 38 and not
      * latitude band S, [32, 40].  N, 01S, 2N, 38S are legal.  0N, 001S, 61N,
      * 38P are illegal.  INV is a special value for which the returned value of
-     * \e is UTMUPS::INVALID.  Throws an error is the zone string is malformed.
+     * \e is UTMUPS::INVALID.
      **********************************************************************/
     static void DecodeZone(const std::string& zonestr, int& zone, bool& northp);
 
     /**
      * Encode a UTM/UPS zone string.
      *
-     * @param[out] zone the UTM zone (zero means UPS).
-     * @param[out] northp hemisphere (true means north, false means south).
+     * @param[in] zone the UTM zone (zero means UPS).
+     * @param[in] northp hemisphere (true means north, false means south).
+     * @exception GeographicErr if \e zone is out of range (see below).
+     * @exception std::bad_alloc if memoy for the string can't be allocated.
      * @return string representation of zone and hemisphere.
      *
      * \e zone must be in the range [UTMUPS::MINZONE, UTMUPS::MAXZONE] = [0,
@@ -277,6 +344,34 @@ namespace GeographicLib {
      **********************************************************************/
     static std::string EncodeZone(int zone, bool northp);
 
+    /**
+     * Decode EPSG.
+     *
+     * @param[in] epsg the EPSG code.
+     * @param[out] zone the UTM zone (zero means UPS).
+     * @param[out] northp hemisphere (true means north, false means south).
+     *
+     * EPSG (European Petroleum Survery Group) codes are a way to refer to many
+     * different projections.  DecodeEPSG decodes those refering to UTM or UPS
+     * projections for the WGS84 ellipsoid.  If the code does not refer to one
+     * of these projections, \e zone is set to UTMUPS::INVALID.  See
+     * http://spatialreference.org/ref/epsg/
+     **********************************************************************/
+    static void DecodeEPSG(int epsg, int& zone, bool& northp) throw();
+
+    /**
+     * Encode zone as EPSG.
+     *
+     * @param[in] zone the UTM zone (zero means UPS).
+     * @param[in] northp hemisphere (true means north, false means south).
+     * @return EPSG code (or -1 if \e zone is not in the range
+     *   [UTMUPS::MINZONE, UTMUPS::MAXZONE] = [0, 60])
+     *
+     * Convert \e zone and \e northp to the corresponding EPSG (European
+     * Petroleum Survery Group) codes
+     **********************************************************************/
+    static int EncodeEPSG(int zone, bool northp) throw();
+
     /**
      * @return shift (meters) necessary to align N and S halves of a UTM zone
      * (10<sup>7</sup>).
