@@ -2,23 +2,26 @@
  * \file Utility.hpp
  * \brief Header for GeographicLib::Utility class
  *
- * Copyright (c) Charles Karney (2011, 2012) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2011-2012) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_UTILITY_HPP)
-#define GEOGRAPHICLIB_UTILITY_HPP \
-  "$Id: 92c92fcb8ea92116fed01909c2611934b708e4cd $"
+#define GEOGRAPHICLIB_UTILITY_HPP 1
 
 #include <GeographicLib/Constants.hpp>
 #include <iomanip>
 #include <vector>
-#include <string>
 #include <sstream>
-#include <algorithm>
 #include <cctype>
 
+#if defined(_MSC_VER)
+// Squelch warnings about constant conditional expressions
+#  pragma warning (push)
+#  pragma warning (disable: 4127)
+#endif
+
 namespace GeographicLib {
 
   /**
@@ -27,7 +30,7 @@ namespace GeographicLib {
    * Example of use:
    * \include example-Utility.cpp
    **********************************************************************/
-  class GEOGRAPHIC_EXPORT Utility {
+  class GEOGRAPHICLIB_EXPORT Utility {
   private:
     static bool gregorian(int y, int m, int d) {
       // The original cut over to the Gregorian calendar in Pope Gregory XIII's
@@ -112,9 +115,8 @@ namespace GeographicLib {
      * @param[in] m the month, Jan = 1, etc. (must be positive).  Default = 1.
      * @param[in] d the day of the month (must be positive).  Default = 1.
      * @param[in] check whether to check the date.
+     * @exception GeographicErr if the date is invalid and \e check is true.
      * @return the sequential day number.
-     *
-     * If \e check is true and the date is invalid an exception is thrown.
      **********************************************************************/
     static int day(int y, int m, int d, bool check) {
       int s = day(y, m, d);
@@ -169,6 +171,7 @@ namespace GeographicLib {
      * @param[out] y the year.
      * @param[out] m the month, Jan = 1, etc.
      * @param[out] d the day of the month.
+     * @exception GeographicErr is \e s is malformed.
      **********************************************************************/
     static void date(const std::string& s, int& y, int& m, int& d) {
       int y1, m1 = 1, d1 = 1;
@@ -207,7 +210,8 @@ namespace GeographicLib {
      * @param[in] y the year (must be positive).
      * @param[in] m the month, Jan = 1, etc. (must be positive).
      * @param[in] d the day of the month (must be positive).
-     * @return the day of the week with Sunday, Monday - Saturday = 0, 1 - 6.
+     * @return the day of the week with Sunday, Monday--Saturday = 0,
+     *   1--6.
      **********************************************************************/
     static int dow(int y, int m, int d) throw() { return dow(day(y, m, d)); }
 
@@ -215,7 +219,8 @@ namespace GeographicLib {
      * Given the sequential day, return the day of the week.
      *
      * @param[in] s the sequential day (must be positive).
-     * @return the day of the week with Sunday, Monday - Saturday = 0, 1 - 6.
+     * @return the day of the week with Sunday, Monday--Saturday = 0,
+     *   1--6.
      **********************************************************************/
     static int dow(int s) throw() {
       return (s + 5) % 7;  // The 5 offset makes day 1 (0001-01-01) a Saturday.
@@ -226,6 +231,7 @@ namespace GeographicLib {
      *
      * @tparam T the type of the argument.
      * @param[in] s the string to be converted.
+     * @exception GeographicErr if \e s can't be interpreted as a date.
      * @return the fractional year.
      *
      * The string is first read as an ordinary number (e.g., 2010 or 2012.5);
@@ -250,10 +256,11 @@ namespace GeographicLib {
      *
      * @tparam T the type of the argument.
      * @param[in] x the value to be converted.
-     * @param[in] p the precision used (default -1).
+     * @param[in] p the precision used (default &minus;1).
+     * @exception std::bad_alloc if memory for the string can't be allocated.
      * @return the string representation.
      *
-     * If \e p >= 0, then the number fixed format is used with p bits of
+     * If \e p &ge; 0, then the number fixed format is used with p bits of
      * precision.  With p < 0, there is no manipulation of the format.
      **********************************************************************/
     template<typename T> static std::string str(T x, int p = -1) {
@@ -270,6 +277,7 @@ namespace GeographicLib {
      *
      * @tparam T the type of the return value.
      * @param[in] s the string to be converted.
+     * @exception GeographicErr is \e s is not readable as a T.
      * @return object of type T
      **********************************************************************/
     template<typename T> static T num(const std::string& s) {
@@ -299,7 +307,8 @@ namespace GeographicLib {
      *
      * @tparam T the type of the return value.
      * @param[in] s the string to be matched.
-     * @return appropriate special value (+/-inf, nan) or 0 is none is found.
+     * @return appropriate special value (&plusmn;&infin;, nan) or 0 if none is
+     *   found.
      **********************************************************************/
     template<typename T> static T nummatch(const std::string& s) {
       if (s.length() < 3)
@@ -308,7 +317,7 @@ namespace GeographicLib {
       t.resize(s.length());
       std::transform(s.begin(), s.end(), t.begin(), (int(*)(int))std::toupper);
       for (size_t i = s.length(); i--;)
-        t[i] = std::toupper(s[i]);
+        t[i] = char(std::toupper(s[i]));
       int sign = t[0] == '-' ? -1 : 1;
       std::string::size_type p0 = t[0] == '-' || t[0] == '+' ? 1 : 0;
       std::string::size_type p1 = t.find_last_not_of('0');
@@ -329,6 +338,7 @@ namespace GeographicLib {
      *
      * @tparam T the type of the return value.
      * @param[in] s the string to be converted.
+     * @exception GeographicErr is \e s is not readable as a fraction of type T.
      * @return object of type T
      **********************************************************************/
     template<typename T> static T fract(const std::string& s) {
@@ -345,14 +355,14 @@ namespace GeographicLib {
      *
      * @param[in] s the string to be searched.
      * @param[in] c the character to look for.
-     * @return the index of the first occurrence character in the string or -1
-     *   is the character is not present.
+     * @return the index of the first occurrence character in the string or
+     *   &minus;1 is the character is not present.
      *
      * \e c is converted to upper case before search \e s.  Therefore, it is
      * intended that \e s should not contain any lower case letters.
      **********************************************************************/
     static int lookup(const std::string& s, char c) throw() {
-      std::string::size_type r = s.find(toupper(c));
+      std::string::size_type r = s.find(char(toupper(c)));
       return r == std::string::npos ? -1 : int(r);
     }
 
@@ -367,6 +377,7 @@ namespace GeographicLib {
      *   (external).
      * @param[out] array the output array of type IntT (internal).
      * @param[in] num the size of the array.
+     * @exception GeographicErr if the data cannot be read.
      **********************************************************************/
     template<typename ExtT, typename IntT, bool bigendp>
       static inline void readarray(std::istream& str,
@@ -413,6 +424,7 @@ namespace GeographicLib {
      * @param[in] str the input stream containing the data of type ExtT
      *   (external).
      * @param[out] array the output vector of type IntT (internal).
+     * @exception GeographicErr if the data cannot be read.
      **********************************************************************/
     template<typename ExtT, typename IntT, bool bigendp>
       static inline void readarray(std::istream& str,
@@ -430,6 +442,7 @@ namespace GeographicLib {
      * @param[out] str the output stream for the data of type ExtT (external).
      * @param[in] array the input array of type IntT (internal).
      * @param[in] num the size of the array.
+     * @exception GeographicErr if the data cannot be written.
      **********************************************************************/
     template<typename ExtT, typename IntT, bool bigendp>
       static inline void writearray(std::ostream& str,
@@ -471,6 +484,7 @@ namespace GeographicLib {
      * @tparam bigendp true if the external storage format is big-endian.
      * @param[out] str the output stream for the data of type ExtT (external).
      * @param[in] array the input vector of type IntT (internal).
+     * @exception GeographicErr if the data cannot be written.
      **********************************************************************/
     template<typename ExtT, typename IntT, bool bigendp>
       static inline void writearray(std::ostream& str,
@@ -484,6 +498,8 @@ namespace GeographicLib {
      * @param[in] line the input line.
      * @param[out] key the key.
      * @param[out] val the value.
+     * @exception std::bad_alloc if memory for the internal strings can't be
+     *   allocated.
      * @return whether a key was found.
      *
      * A # character and everything after it are discarded.  If the results is
@@ -498,4 +514,8 @@ namespace GeographicLib {
 
 } // namespace GeographicLib
 
+#if defined(_MSC_VER)
+#  pragma warning (pop)
+#endif
+
 #endif  // GEOGRAPHICLIB_UTILITY_HPP
