@@ -2,18 +2,17 @@
  * \file Gnomonic.cpp
  * \brief Implementation for GeographicLib::Gnomonic class
  *
- * Copyright (c) Charles Karney (2010, 2011) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2010-2011) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #include <GeographicLib/Gnomonic.hpp>
 
-#define GEOGRAPHICLIB_GNOMONIC_CPP \
-  "$Id: 1abf2f2ebdc8a805d0d205051120809f0c0e6071 $"
-
-RCSID_DECL(GEOGRAPHICLIB_GNOMONIC_CPP)
-RCSID_DECL(GEOGRAPHICLIB_GNOMONIC_HPP)
+#if defined(_MSC_VER)
+// Squelch warnings about potentially uninitialized local variables
+#  pragma warning (disable: 4701)
+#endif
 
 namespace GeographicLib {
 
@@ -67,6 +66,7 @@ namespace GeographicLib {
       // else solve 1/rho(s) = 1/rho with d(1/rho(s))/ds = -1/m^2
       real ds = little ? (m/M - rho) * M * M : (rho - M/m) * m * m;
       s -= ds;
+      // Reversed test to allow escape with NaNs
       if (!(abs(ds) >= eps_ * _a))
         ++trip;
     }
