@@ -2,7 +2,7 @@
  * \file TransverseMercator.cpp
  * \brief Implementation for GeographicLib::TransverseMercator class
  *
- * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2008-2012) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  *
@@ -29,10 +29,10 @@
  *  - evaluating the convergence and scale using the expression for the
  *    projection or its inverse.
  *
- * If the preprocessor variable TM_TX_MAXPOW is set to an integer between 4 and
- * 8, then this specifies the order of the series used for the forward and
- * reverse transformations.  The default value is 6.  (The series accurate to
- * 12th order is given in \ref tmseries.)
+ * If the preprocessor variable GEOGRAPHICLIB_TRANSVERSEMERCATOR_ORDER is set
+ * to an integer between 4 and 8, then this specifies the order of the series
+ * used for the forward and reverse transformations.  The default value is 6.
+ * (The series accurate to 12th order is given in \ref tmseries.)
  *
  * Other equivalent implementations are given in
  *  - http://www.ign.fr/DISPLAY/000/526/702/5267021/NTG_76.pdf
@@ -41,12 +41,6 @@
 
 #include <GeographicLib/TransverseMercator.hpp>
 
-#define GEOGRAPHICLIB_TRANSVERSEMERCATOR_CPP \
-  "$Id: 7b5a1854a015da061b8fdad0a4b35be7e06fcb9a $"
-
-RCSID_DECL(GEOGRAPHICLIB_TRANSVERSEMERCATOR_CPP)
-RCSID_DECL(GEOGRAPHICLIB_TRANSVERSEMERCATOR_HPP)
-
 namespace GeographicLib {
 
   using namespace std;
@@ -214,17 +208,107 @@ namespace GeographicLib {
                           Constants::WGS84_f<real>(),
                           Constants::UTM_k0<real>());
 
+  // Engsager and Poder (2007) use trigonometric series to convert between phi
+  // and phip.
+
+  // Conversion from phi to phip:
+  //
+  //     phip = phi + sum(c[j] * sin(2*j*phi), j, 1, 6)
+  //
+  //       c[1] = - 2 * n
+  //              + 2/3 * n^2
+  //              + 4/3 * n^3
+  //              - 82/45 * n^4
+  //              + 32/45 * n^5
+  //              + 4642/4725 * n^6;
+  //       c[2] =   5/3 * n^2
+  //              - 16/15 * n^3
+  //              - 13/9 * n^4
+  //              + 904/315 * n^5
+  //              - 1522/945 * n^6;
+  //       c[3] = - 26/15 * n^3
+  //              + 34/21 * n^4
+  //              + 8/5 * n^5
+  //              - 12686/2835 * n^6;
+  //       c[4] =   1237/630 * n^4
+  //              - 12/5 * n^5
+  //              - 24832/14175 * n^6;
+  //       c[5] = - 734/315 * n^5
+  //              + 109598/31185 * n^6;
+  //       c[6] =   444337/155925 * n^6;
+
+  // Conversion from phip to phi:
+  //
+  //     phi = phip + sum(d[j] * sin(2*j*phip), j, 1, 6)
+  //
+  //       d[1] =   2 * n
+  //              - 2/3 * n^2
+  //              - 2 * n^3
+  //              + 116/45 * n^4
+  //              + 26/45 * n^5
+  //              - 2854/675 * n^6;
+  //       d[2] =   7/3 * n^2
+  //              - 8/5 * n^3
+  //              - 227/45 * n^4
+  //              + 2704/315 * n^5
+  //              + 2323/945 * n^6;
+  //       d[3] =   56/15 * n^3
+  //              - 136/35 * n^4
+  //              - 1262/105 * n^5
+  //              + 73814/2835 * n^6;
+  //       d[4] =   4279/630 * n^4
+  //              - 332/35 * n^5
+  //              - 399572/14175 * n^6;
+  //       d[5] =   4174/315 * n^5
+  //              - 144838/6237 * n^6;
+  //       d[6] =   601676/22275 * n^6;
+
+  // In order to maintain sufficient relative accuracy close to the pole use
+  //
+  //     S = sum(c[i]*sin(2*i*phi),i,1,6)
+  //     taup = (tau + tan(S)) / (1 - tau * tan(S))
+
+  // taupf and tauf are adapted from TransverseMercatorExact (taup and
+  // taupinv).  tau = tan(phi), taup = sinh(psi)
+  Math::real TransverseMercator::taupf(real tau) const throw() {
+    if (!(abs(tau) < overflow_))
+      return tau;
+    real
+      tau1 = Math::hypot(real(1), tau),
+      sig = sinh( eatanhe(tau / tau1) );
+    return Math::hypot(real(1), sig) * tau - sig * tau1;
+  }
+
+  Math::real TransverseMercator::tauf(real taup) const throw() {
+    if (!(abs(taup) < overflow_))
+      return taup;
+    real
+      // To lowest order in e^2, taup = (1 - e^2) * tau = _e2m * tau; so use
+      // tau = taup/_e2m as a starting guess.  Only 1 iteration is needed for
+      // |lat| < 3.35 deg, otherwise 2 iterations are needed.  If, instead, tau
+      // = taup is used the mean number of iterations increases to 1.99 (2
+      // iterations are needed except near tau = 0).
+      tau = taup/_e2m,
+      stol = tol_ * max(real(1), abs(taup));
+    // min iterations = 1, max iterations = 2; mean = 1.94
+    for (int i = 0; i < numit_; ++i) {
+      real
+        tau1 = Math::hypot(real(1), tau),
+        sig = sinh( eatanhe( tau / tau1 ) ),
+        taupa = Math::hypot(real(1), sig) * tau - sig * tau1,
+        dtau = (taup - taupa) * (1 + _e2m * Math::sq(tau)) /
+        ( _e2m * tau1 * Math::hypot(real(1), taupa) );
+      tau += dtau;
+      if (!(abs(dtau) >= stol))
+        break;
+    }
+    return tau;
+  }
+
   void TransverseMercator::Forward(real lon0, real lat, real lon,
                                    real& x, real& y, real& gamma, real& k)
     const throw() {
-    // Avoid losing a bit of accuracy in lon (assuming lon0 is an integer)
-    if (lon - lon0 > 180)
-      lon -= lon0 + 360;
-    else if (lon - lon0 <= -180)
-      lon -= lon0 - 360;
-    else
-      lon -= lon0;
-    // Now lon in (-180, 180]
+    lon = Math::AngDiff(Math::AngNormalize(lon0), Math::AngNormalize(lon));
     // Explicitly enforce the parity
     int
       latsign = lat < 0 ? -1 : 1,
@@ -262,9 +346,7 @@ namespace GeographicLib {
       real
         c = max(real(0), cos(lam)), // cos(pi/2) might be negative
         tau = tan(phi),
-        secphi = Math::hypot(real(1), tau),
-        sig = sinh( eatanhe(tau / secphi) ),
-        taup = Math::hypot(real(1), sig) * tau - sig * secphi;
+        taup = taupf(tau);
       xip = atan2(taup, c);
       // Used to be
       //   etap = Math::atanh(sin(lam) / cosh(psi));
@@ -281,7 +363,8 @@ namespace GeographicLib {
       // This form has cancelling errors.  This property is lost if cosh(psip)
       // is replaced by 1/cos(phi), even though it's using "primary" data (phi
       // instead of psip).
-      k = sqrt(_e2m + _e2 * Math::sq(cos(phi))) * secphi / Math::hypot(taup, c);
+      k = sqrt(_e2m + _e2 * Math::sq(cos(phi))) * Math::hypot(real(1), tau)
+        / Math::hypot(taup, c);
     } else {
       xip = Math::pi<real>()/2;
       etap = 0;
@@ -296,7 +379,7 @@ namespace GeographicLib {
     //   zeta = xi + i*eta
     //   zeta' = xi' + i*eta'
     //
-    // The conversion from conformal to rectifying latitude can be expresses as
+    // The conversion from conformal to rectifying latitude can be expressed as
     // a series in _n:
     //
     //   zeta = zeta' + sum(h[j-1]' * sin(2 * j * zeta'), j = 1..maxpow_)
@@ -450,25 +533,7 @@ namespace GeographicLib {
       // Use Newton's method to solve for tau
       real
         taup = sin(xip)/r,
-        // To lowest order in e^2, taup = (1 - e^2) * tau = _e2m * tau; so use
-        // tau = taup/_e2m as a starting guess.  Only 1 iteration is needed for
-        // |lat| < 3.35 deg, otherwise 2 iterations are needed.  If, instead,
-        // tau = taup is used the mean number of iterations increases to 1.99
-        // (2 iterations are needed except near tau = 0).
-        tau = taup/_e2m,
-        stol = tol_ * max(real(1), abs(taup));
-      // min iterations = 1, max iterations = 2; mean = 1.94
-      for (int i = 0; i < numit_; ++i) {
-        real
-          tau1 = Math::hypot(real(1), tau),
-          sig = sinh( eatanhe( tau / tau1 ) ),
-          taupa = Math::hypot(real(1), sig) * tau - sig * tau1,
-          dtau = (taup - taupa) * (1 + _e2m * Math::sq(tau)) /
-          ( _e2m * tau1 * Math::hypot(real(1), taupa) );
-        tau += dtau;
-        if (!(abs(dtau) >= stol))
-          break;
-      }
+        tau = tauf(taup);
       phi = atan(tau);
       gamma += atan(tanx(xip) * tanh(etap)); // Krueger p 19 (31)
       // Note cos(phi') * cosh(eta') = r
@@ -484,13 +549,7 @@ namespace GeographicLib {
     if (backside)
       lon = 180 - lon;
     lon *= etasign;
-    // Avoid losing a bit of accuracy in lon (assuming lon0 is an integer)
-    if (lon + lon0 >= 180)
-      lon += lon0 - 360;
-    else if (lon + lon0 < -180)
-      lon += lon0 + 360;
-    else
-      lon += lon0;
+    lon = Math::AngNormalize(lon + Math::AngNormalize(lon0));
     gamma /= Math::degree<real>();
     if (backside)
       gamma = 180 - gamma;
