@@ -6,7 +6,7 @@
  * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  *
- * The relevant section of Lee's paper is part V, pp 67&ndash;101,
+ * The relevant section of Lee's paper is part V, pp 67--101,
  * <a href="http://dx.doi.org/10.3138/X687-1574-4325-WM62">Conformal
  * Projections Based On Jacobian Elliptic Functions</a>.
  *
@@ -41,12 +41,6 @@
 
 #include <GeographicLib/TransverseMercatorExact.hpp>
 
-#define GEOGRAPHICLIB_TRANSVERSEMERCATOREXACT_CPP \
-  "$Id: 125a2d11919018a84fb0c09ea2e77011a35a4a2d $"
-
-RCSID_DECL(GEOGRAPHICLIB_TRANSVERSEMERCATOREXACT_CPP)
-RCSID_DECL(GEOGRAPHICLIB_TRANSVERSEMERCATOREXACT_HPP)
-
 namespace GeographicLib {
 
   using namespace std;
@@ -67,7 +61,6 @@ namespace GeographicLib {
     , _mu(_f * (2 - _f))        // e^2
     , _mv(1 - _mu)              // 1 - e^2
     , _e(sqrt(_mu))
-    , _ep2(_mu / _mv)           // e^2 / (1 - e^2)
     , _extendp(extendp)
     , _Eu(_mu)
     , _Ev(_mv)
@@ -350,9 +343,9 @@ namespace GeographicLib {
   }
 
   void TransverseMercatorExact::Scale(real tau, real /*lam*/,
-                                       real snu, real cnu, real dnu,
-                                       real snv, real cnv, real dnv,
-                                       real& gamma, real& k) const throw() {
+                                      real snu, real cnu, real dnu,
+                                      real snv, real cnv, real dnv,
+                                      real& gamma, real& k) const throw() {
     real sec2 = 1 + Math::sq(tau);    // sec(phi)^2
     // Lee 55.12 -- negated for our sign convention.  gamma gives the bearing
     // (clockwise from true north) of grid north
@@ -378,18 +371,11 @@ namespace GeographicLib {
   void TransverseMercatorExact::Forward(real lon0, real lat, real lon,
                                         real& x, real& y, real& gamma, real& k)
     const throw() {
-    // Avoid losing a bit of accuracy in lon (assuming lon0 is an integer)
-    if (lon - lon0 > 180)
-      lon -= lon0 + 360;
-    else if (lon - lon0 <= -180)
-      lon -= lon0 - 360;
-    else
-      lon -= lon0;
-    // Now lon in (-180, 180]
+    lon = Math::AngDiff(Math::AngNormalize(lon0), Math::AngNormalize(lon));
     // Explicitly enforce the parity
     int
-      latsign = !_extendp && lat < 0 ? -1 : 1,
-      lonsign = !_extendp && lon < 0 ? -1 : 1;
+      latsign = (!_extendp && lat < 0) ? -1 : 1,
+      lonsign = (!_extendp && lon < 0) ? -1 : 1;
     lon *= lonsign;
     lat *= latsign;
     bool backside = !_extendp && lon > 90;
@@ -425,11 +411,16 @@ namespace GeographicLib {
     y = xi * _a * _k0 * latsign;
     x = eta * _a * _k0 * lonsign;
 
-    // Recompute (tau, lam) from (u, v) to improve accuracy of Scale
-    zeta(u, snu, cnu, dnu, v, snv, cnv, dnv, tau, lam);
-    tau=taupinv(tau);
-    Scale(tau, lam, snu, cnu, dnu, snv, cnv, dnv, gamma, k);
-    gamma /= Math::degree<real>();
+    if (lat == 90) {
+      gamma = lon;
+      k = 1;
+    } else {
+      // Recompute (tau, lam) from (u, v) to improve accuracy of Scale
+      zeta(u, snu, cnu, dnu, v, snv, cnv, dnv, tau, lam);
+      tau=taupinv(tau);
+      Scale(tau, lam, snu, cnu, dnu, snv, cnv, dnv, gamma, k);
+      gamma /= Math::degree<real>();
+    }
     if (backside)
       gamma = 180 - gamma;
     gamma *= latsign * lonsign;
@@ -472,24 +463,20 @@ namespace GeographicLib {
       phi = atan(tau);
       lat = phi / Math::degree<real>();
       lon = lam / Math::degree<real>();
+      Scale(tau, lam, snu, cnu, dnu, snv, cnv, dnv, gamma, k);
+      gamma /= Math::degree<real>();
     } else {
       tau = overflow_;
       phi = Math::pi<real>()/2;
       lat = 90;
-      lon = lam = 0;
+      lon = lam = gamma = 0;
+      k = 1;
     }
-    Scale(tau, lam, snu, cnu, dnu, snv, cnv, dnv, gamma, k);
-    gamma /= Math::degree<real>();
+
     if (backside)
       lon = 180 - lon;
     lon *= lonsign;
-    // Avoid losing a bit of accuracy in lon (assuming lon0 is an integer)
-    if (lon + lon0 >= 180)
-      lon += lon0 - 360;
-    else if (lon + lon0 < -180)
-      lon += lon0 + 360;
-    else
-      lon += lon0;
+    lon = Math::AngNormalize(lon + Math::AngNormalize(lon0));
     lat *= latsign;
     if (backside)
       y = 2 * _Eu.E() - y;
